"""
Copyright 2008-2014 by Aerospike, Inc. All rights reserved.
THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE. THE COPYRIGHT NOTICE
ABOVE DOES NOT EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION.
"""


# -*- coding: utf-8 -*-
from constants import AMC_PATH, ENTERPRISE_FLAG_FILE
from constants import AMC_CONFIG_PATH
from constants import SECRET_KEY, ALLOWED_ROLES
import sys, datetime, os.path
sys.path.insert(1,'%s/site-packages/' % AMC_PATH)
sys.path.append(str(os.path.dirname(AMC_CONFIG_PATH)))

from flask import Flask
from libs.flask_compress import Compress
from utilities import background_tasks, config_manager
import traceback

app = Flask(__name__)
app.config["UPDATE_INTERVAL"] = int(config_manager.get_update_interval())
app.secret_key = SECRET_KEY
app.config["COMPRESS_DEBUG"] = True
app.config["COMPRESS_MIMETYPES"] = ["text/javascript", "application/javascript", "application/x-javascript", "image/png", "text/html", "text/css"]

try:
    from gunicorn_config import certfile, keyfile
    if os.path.exists(certfile) and os.path.exists(keyfile):
        app.config['SESSION_COOKIE_SECURE'] = True
        app.config['SESSION_COOKIE_HTTPONLY'] = False
except ImportError:
    app.config['SESSION_COOKIE_SECURE'] = False
    app.config['SESSION_COOKIE_HTTPONLY'] = True

Compress(app)

from flask import json, request, render_template
from controllers import cluster_controller, namespace_controller
from controllers import config_controller, xdr_controller
from controllers import node_controller, udf_controller
from utilities.cluster_manager import ClusterManager
from utilities.user_manager import UserManager
from flask_login import (LoginManager, login_required, login_user, logout_user, current_user, session)
from itsdangerous import URLSafeTimedSerializer
from models.user import User
from helpers import encryption
import eventlet

ansible_present = True
try:
    import ansible.runner
    from ansible.errors import AnsibleError
except ImportError:
    ansible_present = False

cluster_manager = ClusterManager()

user_manager = UserManager()

# Login_serializer used to encryt and decrypt the cookie token for the remember
# me option of flask-login
login_serializer = URLSafeTimedSerializer(app.secret_key)

#Flask-Login Login Manager
login_manager = LoginManager()


@login_manager.user_loader
def load_user(userid):
    """
    Flask-Login user_loader callback.
    The user_loader function asks this function to get a User Object or return
    None based on the userid.
    The userid was stored in the session environment by Flask-Login.
    user_loader stores the returned User object in current_user during every
    flask request.
    """
    password, host, roles, en_pw = None, None, [], None
    if 'password' in session:
        password = session.get('password', '')
    if 'host' in session:
        host = session.get('host', '')
    if 'roles' in session:
        roles = session.get('roles', [])
    if 'en_pw' in session:
        en_pw = session.get('en_pw', '')
    plane_password = User.get_decrypted_password(str(en_pw))

    return User.get(userid, password, host=host, roles=roles, pl_password=plane_password)


@login_manager.token_loader
def load_token(token):
    """
    Flask-Login token_loader callback.
    The token_loader function asks this function to take the token that was
    stored on the users computer process it to check if its valid and then
    return a User Object if its valid or None if its not valid.
    """

    #The Token itself was generated by User.get_auth_token.  So it is up to
    #us to known the format of the token data itself.

    #The Token was encrypted using itsdangerous.URLSafeTimedSerializer which
    #allows us to have a max_age on the token itself.  When the cookie is stored
    #on the users computer it also has a exipry date, but could be changed by
    #the user, so this feature allows us to enforce the exipry date of the token
    #server side and not rely on the users cookie to expire.

    max_age = app.config["REMEMBER_COOKIE_DURATION"].total_seconds()

    #Decrypt the Security Token, data = [username, hashpass]
    data = login_serializer.loads(token, max_age=max_age)


    #Find the User
    user = User.get(data[0])

    #Check Password and return user or None
    if user and data[1] == user.password:
        return user
    return None

# URLs for static contents


@app.route("/", methods=["GET"])
def onepage():

    response = app.make_response(render_template('onepage.html'))
    response.headers.add('amc-page', 'homepage')
    return response


# URLs for apis

@app.route("/logout/<cluster>", methods=["GET"])
@login_required
@cluster_manager.check_cluster_validity()
def logout_page(cluster):

    #cluster.remove_user(current_user)
    logout_user()
    response = {'status': 'success'}
    return json.dumps(response)


@app.route("/login/<node_address>", methods=["POST"])
def login_page(node_address):

    from libs import citrusleaf as cl

    try:
        ip, port = node_address.split(":")
    except:
        return dict(error="Unable to connect - %s" % node_address)

    try:
        if 'username' in request.form and 'password' in request.form:
            user = User( str(request.form['username']),
                         cl.hashpassword( str(request.form['password'] )),
                         pl_password=request.form["password"] )
        else:
            user = User.get("admin")

        # If we found a user based on username then compare that the submitted
        # password matches the password in the database.  The password is stored
        # is a slated hash format, so you must hash the password before comparing it.

        if user and user.authenticate( str(ip),
                                       str(port)):

            roles = user.get_roles(str(ip), str(port))

            roles.sort()

            cluster_info = cluster_manager.get_cluster_info(node_address,
                                    user=user.id, password=user.password)

            if isinstance(roles, list) and set(ALLOWED_ROLES).issubset(roles):
                cluster_id = cluster_info.get('cluster_id', '')
                login_user(user, cluster_id=cluster_id, ip=ip, port=port)
                response = {'status': 'success', 'cluster_id': cluster_info.get('cluster_id', '')}
            else:
                response = {'status': 'failure', 'error': "Don't have enough perimission to login."}
        else:
            response = {'status': 'failure', 'error': 'Invalid username or password.'}
    except Exception as e:
        traceback.print_exc(file=sys.stderr)
        response = {'status': 'failure', 'error': str(e)}
        raise e
    return json.dumps(response)


@app.route("/change_password", methods=["POST"])
@login_required
def change_password():

    if 'username' in request.form:
        user = User.get(request.form['username'])
    else:
        user = User.get("admin")

    if user and encryption.hash_pass(request.form['old_password'], app.secret_key) == user.password:
        current_users = UserManager()
        status = current_users.change_password(user.id, encryption.hash_pass(request.form['new_password'], app.secret_key))
        if status == "success":
            response = {'status': "success"}
            return json.dumps(response)
    response = {'status': "failure"}
    return json.dumps(response)


@app.route("/get-current-user", methods=["GET"])
def get_current_user():

    username = current_user.get_id()

    response = dict(username=username)
    response['cluster_id'] = session.get('cluster_id', None)

    return json.dumps(response)


@app.route("/set-update-interval/<cluster>", methods=["POST"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("sys-admin")
def set_update_interval(cluster):

    if int(request.form['update_interval']) < 1 or int(request.form['update_interval']) > 10:
        response = dict(status="invalid")
    else:
        response = config_manager.set_update_interval(request.form['update_interval'])
    return json.dumps(response)


@app.route("/get_amc_version", methods=["GET"])
def get_amc_version():

    try:
        f = open('/opt/amc/amc_version', 'r')
        amc_version = f.read()
        f.close()
    except:
        amc_version = "Error reading AMC version"
    amc_type = "enterprise" if os.path.isfile(ENTERPRISE_FLAG_FILE) else "community"
    return json.dumps(dict(amc_version=amc_version.strip(), amc_type=amc_type))


@app.route("/aerospike/service/clusters/get-cluster-id/<node_address>", methods=["GET"])
def get_cluster_id(node_address):

    user = session.get('user_id', '')
    password = session.get('password', '')

    response = cluster_manager.get_cluster_info(node_address, user=user, password=password)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_cluster_info(cluster):

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = cluster_controller.get_info(cluster, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/basic", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_cluster_basic_info(cluster):

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()
    response = cluster_controller.get_basic_info(cluster, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<node_list>", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_node_info(cluster, node_list):

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = node_controller.get_info(cluster, node_list,
                                    user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/allstats", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_node_stats(cluster, address):

    stats_type = request.args.get("type", "all")

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = node_controller.get_stats(cluster, address, stats_type,
                                user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/allconfig", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_node_config(cluster, address):

    response = node_controller.get_all_config(cluster, address)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<node_list>/setconfig", methods=["POST"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("sys-admin")
def set_node_config(cluster, node_list):

    edit_parameters = request.form

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = config_controller.set_node_config(cluster, node_list, edit_parameters,
                                                user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/nodes/<node_list>",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_namespace_info_from_node(cluster, namespace, node_list):

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = namespace_controller.get_namespace_info_per_node(cluster, str(namespace), node_list,
                                            user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/nodes/<node>/allstats",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_namespace_stats_from_node(cluster, namespace, node):

    stats_type = request.args.get("type", "all")

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = namespace_controller.get_stats(cluster, str(namespace), node, stats_type,
                                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace_list>", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_namespace_info(cluster, namespace_list):

    node_addresses = request.args.get('nodes', None)

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = namespace_controller.get_namespace_info(cluster, str(namespace_list), node_addresses,
                                                user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/nodes/<node>/allconfig",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_namespace_config(cluster, namespace, node):

    response = namespace_controller.get_all_config(cluster, str(namespace), node)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace_name>/nodes/<node_list>/setconfig", methods=["POST"])
@login_required
@cluster_manager.check_cluster_validity()
def set_namespace_config(cluster, namespace_name, node_list):

    edit_parameters = request.form

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = config_controller.set_namespace_config(cluster, namespace_name, node_list, edit_parameters, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sindexes/<sindex>/nodes/<node>/allstats",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_secondary_index_stats_from_node(cluster, namespace, sindex, node):

    stats_type = request.args.get("type", "all")

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = namespace_controller.get_index_stats(cluster, str(namespace), str(sindex), node, stats_type, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sindexes",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_secondary_indexes_by_namespace(cluster, namespace):

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = cluster_controller.get_indexes_by_namespace(cluster, str(namespace),
                                            user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/sets",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_sets_by_namespace(cluster, namespace):

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = cluster_controller.get_sets_by_namespace(cluster, str(namespace),
                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/namespaces/<namespace>/storage",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_storage_info(cluster, namespace):

    response = namespace_controller.get_storage_info(cluster, str(namespace))
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node_list>", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_xdr_info(cluster, xdr_port, node_list):

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = xdr_controller.get_info(cluster, node_list, xdr_port,
                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node>/allstats",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_xdr_stats_from_node(cluster, xdr_port, node):

    stats_type = request.args.get("type", "all")

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = xdr_controller.get_stats(cluster, node, xdr_port, stats_type,
                                    user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node>/allconfig",
    methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_xdr_config(cluster, xdr_port, node):

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = xdr_controller.get_all_config(cluster, node, xdr_port,
                                user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/xdr/<xdr_port>/nodes/<node_list>/setconfig",
    methods=["POST"])
@login_required
@cluster_manager.check_cluster_validity()
def set_xdr_config(cluster, xdr_port, node_list):

    edit_parameters = request.form

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = config_controller.set_xdr_config(cluster, node_list, xdr_port, edit_parameters,
                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/throughput", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_throughput(cluster):

    node_addresses = request.args.get('nodes', None)

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = cluster_controller.get_throughput(cluster, node_addresses,
                                            user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/throughput_history", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_throughput_history(cluster):

    start_time = request.args.get('start_time', None)
    end_time = request.args.get('end_time', None)
    if start_time is not None:
        start_time = int(start_time)
    if end_time is not None:
        end_time = int(end_time)
    node_addresses = request.args.get('nodes', None)

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = cluster_controller.get_throughput_history(cluster,
                                        node_addresses, start_time, end_time,
                                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/latency/<node_list>", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_latency_by_node(cluster, node_list):

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = node_controller.get_latency(cluster, str(node_list),
                        user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/latency_history/<node>", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_latency_history_by_node(cluster, node):

    start_time = request.args.get('start_time', None)
    end_time = request.args.get('end_time', None)

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = node_controller.get_latency_history(cluster, str(node), start_time, end_time,
                                            user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters", methods=["GET"])
def get_clusters_info():

    response = cluster_controller.get_clusters(cluster_manager)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/alerts", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_cluster_alerts(cluster):

    last_id = request.args.get('last_id', 0)

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = cluster_controller.get_alerts(cluster, last_id, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/jobs/nodes/<node>", methods=["GET"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("sys-admin")
def get_scan_jobs(cluster, node):

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = udf_controller.get_jobs(cluster, node, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/udfs", methods=["GET"])
@cluster_manager.check_cluster_validity()
def get_udfs(cluster):

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = udf_controller.get_udfs(cluster, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_on", methods=["POST"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("sys-admin")
def switch_node_on(cluster, address):

    if ansible_present is True:

        user, password = current_user.get_id(), None
        if user:
            password = current_user.get_password()

        if "username" in request.form and "password" in request.form:
            response = node_controller.change_node_status("on", cluster, address, request.form['username'], request.form['password'], usr=user, passwd=password or None)
        elif "username" in request.form:
            response = node_controller.change_node_status("on", cluster, address, request.form['username'], usr=user, passwd=password or None)
        else:
            response = node_controller.change_node_status("on", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_off", methods=["POST"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("sys-admin")
def switch_node_off(cluster, address):

    if ansible_present is True:

        user, password = current_user.get_id(), None
        if user:
            password = current_user.get_password()

        if "username" in request.form and "password" in request.form:
            response = node_controller.change_node_status("off", cluster, address, request.form['username'], request.form['password'], usr=user, passwd=password or None)
        elif "username" in request.form:
            response = node_controller.change_node_status("off", cluster, address, request.form['username'], usr=user, passwd=password or None)
        else:
            response = node_controller.change_node_status("off", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_xdr_on", methods=["POST"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("sys-admin")
def switch_xdr_on(cluster, address):

    if ansible_present is True:
        user, password = current_user.get_id(), None
        if user:
            password = current_user.get_password()

        if "username" in request.form and "password" in request.form:
            response = xdr_controller.change_xdr_status("on", cluster, address, request.form['username'], request.form['password'], usr=user, passwd=password or None)
        elif "username" in request.form:
            response = xdr_controller.change_xdr_status("on", cluster, address, request.form['username'], usr=user, passwd=password or None)
        else:
            response = xdr_controller.change_xdr_status("on", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/nodes/<address>/switch_xdr_off", methods=["POST"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("sys-admin")
def switch_xdr_off(cluster, address):

    if ansible_present is True:

        user, password = current_user.get_id(), None
        if user:
            password = current_user.get_password()

        if "username" in request.form and "password" in request.form:
            response = xdr_controller.change_xdr_status("off", cluster, address, request.form['username'], request.form['password'], usr=user, passwd=password or None)
        elif "username" in request.form:
            response = xdr_controller.change_xdr_status("off", cluster, address, request.form['username'], usr=user, passwd=password or None)
        else:
            response = xdr_controller.change_xdr_status("off", cluster, address, usr=user, passwd=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/add_node", methods=["POST"])
@cluster_manager.check_cluster_validity()
def add_node(cluster):

    address = request.form['address']

    user, password = current_user.get_id(), None
    if user:
        password = current_user.get_password()

    response = cluster_controller.add_node(cluster, address, user=user, password=password or None)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/remove_node", methods=["POST"])
@cluster_manager.check_cluster_validity()
def remove_node(cluster):

    address = request.form['address']
    response = cluster_controller.remove_node(cluster, address)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/initiate_backup", methods=["POST"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("read", "read-write")
def initiate_backup(cluster):

    if ansible_present is True:

        user, password, pl_password = current_user.get_id(), None, None
        if user:
            password = current_user.get_password()
            pl_password= current_user.get_plain_password()

        if "username" in request.form and "password" in request.form:
            response = cluster_controller.initiate_backup(cluster,
                                                   request.form['namespace'],
                                                   request.form['destination_node_address'],
                                                   request.form['destination_location'],
                                                   request.form['sets'],
                                                   request.form['only_metadata'],
                                                   request.form['scan_priority'],
                                                   request.form['terminate_on_change'],
                                                   request.form['username'],
                                                   request.form['password'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password
                                                   )
        elif "username" in request.form:
            response = cluster_controller.initiate_backup(cluster,
                                                   request.form['namespace'],
                                                   request.form['destination_node_address'],
                                                   request.form['destination_location'],
                                                   request.form['sets'],
                                                   request.form['only_metadata'],
                                                   request.form['scan_priority'],
                                                   request.form['terminate_on_change'],
                                                   request.form['username'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password                                                )
        else:
            response = cluster_controller.initiate_backup(cluster,
                                                   request.form['namespace'],
                                                   request.form['destination_node_address'],
                                                   request.form['destination_location'],
                                                   request.form['sets'],
                                                   request.form['only_metadata'],
                                                   request.form['scan_priority'],
                                                   request.form['terminate_on_change'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password
                                                   )
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_backup_progress", methods=["GET"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("read", "read-write")
def get_backup_progress(cluster):

    if ansible_present is True:
        response = cluster_controller.get_backup_progress(cluster)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_successful_backups", methods=["GET"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("read", "read-write")
def get_successful_backups(cluster):

    response = cluster_controller.get_successful_backups(cluster)
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_available_backups", methods=["POST"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("read", "read-write")
def get_available_backups(cluster):

    if ansible_present is True:
        response = cluster_controller.get_available_backups(cluster,
        request.form['source_node_address'], request.form['source_location'],
        request.form['username'], request.form['password']
        )
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/initiate_restore", methods=["POST"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("read-write")
def initiate_restore(cluster):

    if ansible_present is True:

        user, password, pl_password = current_user.get_id(), None, None
        if user:
            password = current_user.get_password()
            pl_password= current_user.get_plain_password()

        if "username" in request.form and "password" in request.form:
            response = cluster_controller.initiate_restore(cluster,
                                                   request.form['source_node_address'],
                                                   request.form['source_location'],
                                                   request.form['threads'],
                                                   request.form['namespace'],
                                                   request.form['missing_records_only'],
                                                   request.form['ignore_generation_num'],
                                                   request.form['username'],
                                                   request.form['password'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password
                                                   )
        elif "username" in request.form:
            response = cluster_controller.initiate_restore(cluster,
                                                   request.form['source_node_address'],
                                                   request.form['source_location'],
                                                   request.form['threads'],
                                                   request.form['namespace'],
                                                   request.form['missing_records_only'],
                                                   request.form['ignore_generation_num'],
                                                   request.form['username'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password
                                                   )
        else:
            response = cluster_controller.initiate_restore(cluster,
                                                   request.form['source_node_address'],
                                                   request.form['source_location'],
                                                   request.form['threads'],
                                                   request.form['namespace'],
                                                   request.form['missing_records_only'],
                                                   request.form['ignore_generation_num'],
                                                   usr=user,
                                                   passwd=password or None,
                                                   pl_password=pl_password
                                                   )
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.route("/aerospike/service/clusters/<cluster>/get_restore_progress", methods=["GET"])
@cluster_manager.check_cluster_validity()
@user_manager.allowed_for_roles("read-write")
def get_restore_progress(cluster):

    if ansible_present is True:

        user, password = current_user.get_id(), None
        if user:
            password = current_user.get_password()

        response = cluster_controller.get_restore_progress(cluster, user=user, password=password or None)
    else:
        response = dict(status="Failure", error="Ansible Not Found")
    return json.dumps(response)


@app.after_request
def after_request(response):

    response.headers.add('Access-Control-Allow-Origin', '*')
    response.headers.add('Access-Control-Allow-Credentials', 'true')
    response.headers.add('Access-Control-Max-Age', 10)
    response.headers.add('Access-Control-Allow-Methods', 'GET')
    response.headers.add('Cache-Control',
        'must-revalidate, post-check=0, pre-check=0')
    content_type = response.headers.get('Content-Type')
    if content_type.find("text/html") != -1:
        try:
            homepage = response.headers['amc-page']
        except:
            response.headers['Content-Type'] = "application/json"
            response.headers['Cache-Control'] += ', no-cache'
            return response
    expiry_time = datetime.datetime.utcnow() + datetime.timedelta(30)
    response.headers["Expires"] = expiry_time.strftime("%a, %d %b %Y %H:%M:%S GMT")
    return response

@app.route("/aerospike/service/clusters/<cluster>/get_user_roles", methods=["GET"])
@login_required
@cluster_manager.check_cluster_validity()
def get_user_roles(cluster):

    roles = current_user.get_roles(cluster.get_seed_ip(), cluster.get_seed_port())
    return json.dumps(dict(rolelist=roles))


app.debug = True
app.config["REMEMBER_COOKIE_DURATION"] = datetime.timedelta(days=14)

# Configuring an application with the flask-login
# refer https://flask-login.readthedocs.org/en/latest/
login_manager.init_app(app)

e = eventlet.spawn(background_tasks.perform_scheduled_tasks, cluster_manager, app)
try:
    f = open('/opt/amc/amc_version', 'r')
    amc_version = f.read()
    f.close()
except:
    amc_version = "Error reading AMC version"
sys.stderr.write("AMC Version: %s\n" % amc_version)

if __name__ == "__main__":

    #Change the duration of how long the Remember Cookie is valid on the users
    #computer.  This can not really be trusted as a user can edit it.
    app.config["REMEMBER_COOKIE_DURATION"] = datetime.timedelta(days=14)

    #Tell the login manager where to redirect users to display the login page
    #login_manager.login_view = "/login/"

    #Setup the login manager.
    login_manager.init_app(app)

    app.run(host='0.0.0.0', port=5001)

